=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2008-02-27 02:40:57 +0000
+++ loggerhead/history.py	2008-02-27 01:55:39 +0000
@@ -253,12 +253,8 @@
     @with_branch_lock
     def out_of_date(self):
         # the branch may have been upgraded on disk, in which case we're stale.
-        newly_opened = bzrlib.branch.Branch.open(self._branch.base)
         if self._branch.__class__ is not \
-               newly_opened.__class__:
-            return True
-        if self._branch.repository.__class__ is not \
-               newly_opened.repository.__class__:
+               bzrlib.branch.Branch.open(self._branch.base).__class__:
             return True
         return self._branch.last_revision() != self._last_revid
 
@@ -657,13 +653,20 @@
     @with_branch_lock
     @with_bzrlib_read_lock
     def get_changes_uncached(self, revid_list):
-        revid_list = filter(lambda revid: not bzrlib.revision.is_null(revid),
-                            revid_list)
-        repo = self._branch.repository
-        rev_list = repo.get_revisions(
-            repo.get_graph().get_parent_map(revid_list))
+        # Because we may loop and call get_revisions multiple times (to throw
+        # out dud revids), we grab the bzrlib read lock.
+        while True:
+            try:
+                rev_list = self._branch.repository.get_revisions(revid_list)
+            except (KeyError, bzrlib.errors.NoSuchRevision), e:
+                # this sometimes happens with arch-converted branches.
+                # i don't know why. :(
+                self.log.debug('No such revision (skipping): %s', e)
+                revid_list.remove(e.revision)
+            else:
+                break
 
-        return [self._change_from_revision(rev) for rev in rev_list]
+        return [self._change_from_revision(rev) for rev in rev_list]        
 
     def _get_deltas_for_revisions_with_trees(self, revisions):
         """Produce a list of revision deltas.

=== modified file 'loggerhead/tests/test_simple.py'
--- loggerhead/tests/test_simple.py	2008-02-27 02:41:36 +0000
+++ loggerhead/tests/test_simple.py	2008-02-27 01:55:39 +0000
@@ -32,7 +32,7 @@
         self.tearDown()
 
     def setUp(self):
-        #logging.basicConfig(level=logging.DEBUG)
+        logging.basicConfig(level=logging.DEBUG)
         self.bzrbranch = None
         self.old_bzrhome = None
 

